@@ -1,4 +1,4 @@
-/* Copyright (c) 1993
+/* Copyright (c) 1993-2000
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
@@ -55,6 +55,7 @@ extern struct event logflushev;
 extern int log_flush, logtstamp_after;
 extern int ZombieKey_destroy, ZombieKey_resurrect;
 extern struct layer *flayer;
+extern int maxusercount;
 
 #if defined(TIOCSWINSZ) || defined(TIOCGWINSZ)
 extern struct winsize glwz;
@@ -66,7 +67,7 @@ extern int aixhack;
 
 static void WinProcess __P((char **, int *));
 static void WinRedisplayLine __P((int, int, int, int));
-static void WinClearLine __P((int, int, int));
+static void WinClearLine __P((int, int, int, int));
 static int  WinRewrite __P((int, int, int, struct mchar *, int));
 static int  WinResize __P((int, int));
 static void WinRestore __P((void));
@@ -112,8 +113,9 @@ struct NewWindow nwin_undef   =
   -1,		/* wrap */
   -1,		/* logging */
   -1,		/* slowpaste */
-  -1, 		/* c1 */
   -1, 		/* gr */
+  -1, 		/* c1 */
+  -1, 		/* bce */
   -1, 		/* kanji */
   (char *)0,	/* hstatus */
   (char *)0	/* charset */
@@ -136,8 +138,9 @@ struct NewWindow nwin_default =
   1,		/* wrap */
   0,		/* logging */
   0,		/* slowpaste */
-  1,		/* c1 */
   0,		/* gr */
+  1,		/* c1 */
+  0,		/* bce */
   0,		/* kanji */
   (char *)0,	/* hstatus */
   (char *)0	/* charset */
@@ -168,8 +171,9 @@ struct NewWindow *def, *new, *res;
   COMPOSE(wrap);
   COMPOSE(Lflag);
   COMPOSE(slow);
-  COMPOSE(c1);
   COMPOSE(gr);
+  COMPOSE(c1);
+  COMPOSE(bce);
   COMPOSE(kanji);
   COMPOSE(hstatus);
   COMPOSE(charset);
@@ -394,6 +398,10 @@ struct mchar *rend;
   p = fore->w_mlines[y].attr + x1;
 #ifdef FONT
   f = fore->w_mlines[y].font + x1;
+# ifdef KANJI
+  if (rend->font == KANJI)
+    return EXPENSIVE;
+# endif
 #endif
 #ifdef COLOR
   c = fore->w_mlines[y].color + x1;
@@ -417,11 +425,12 @@ struct mchar *rend;
 }
 
 static void
-WinClearLine(y, xs, xe)
-int y, xs, xe;
+WinClearLine(y, xs, xe, bce)
+int y, xs, xe, bce;
 {
   fore = (struct win *)flayer->l_data;
-  LClearLine(flayer, y, xs, xe, &fore->w_mlines[y]);
+  debug3("WinClearLine %d %d-%d\n", y, xs, xe);
+  LClearLine(flayer, y, xs, xe, bce, &fore->w_mlines[y]);
 }
 
 static int
@@ -451,6 +460,7 @@ WinRestore()
       InsertMode(fore->w_insert);
       ReverseVideo(fore->w_revvid);
       CursorVisibility(fore->w_curinv ? -1 : fore->w_curvvis);
+      MouseMode(fore->w_mouse);
     }
 }
 
@@ -624,11 +634,27 @@ struct NewWindow *newwin;
   if (nwin.hstatus)
     p->w_hstatus = SaveStr(nwin.hstatus);
   p->w_monitor = nwin.monitor;
+#ifdef MULTIUSER
+  if (p->w_monitor == MON_ON)
+    {
+      /* always tell all users */
+      for (i = 0; i < maxusercount; i++)
+	ACLBYTE(p->w_mon_notify, i) |= ACLBIT(i);
+    }
+#endif
   /*
    * defsilence by Lloyd Zusman (zusman_lloyd@jpmorgan.com)
    */
   p->w_silence = nwin.silence;
   p->w_silencewait = SilenceWait;
+#ifdef MULTIUSER
+  if (p->w_silence == SILENCE_ON)
+    {
+      /* always tell all users */
+      for (i = 0; i < maxusercount; i++)
+	ACLBYTE(p->w_lio_notify, i) |= ACLBIT(i);
+    }
+#endif
 #ifdef COPY_PASTE
   p->w_slowpaste = nwin.slow;
 #else
@@ -638,6 +664,8 @@ struct NewWindow *newwin;
   p->w_norefresh = 0;
   strncpy(p->w_tty, TtyName, MAXSTR - 1);
 
+#if 0
+  /* XXX Fixme display resize */
   if (ChangeWindowSize(p, display ? D_defwidth : 80,
 		       display ? D_defheight : 24, 
 		       nwin.histheight))
@@ -645,10 +673,20 @@ struct NewWindow *newwin;
       FreeWindow(p);
       return -1;
     }
+#else
+  if (ChangeWindowSize(p, display ? D_forecv->c_xe - D_forecv->c_xs + 1: 80,
+		       display ? D_forecv->c_ye - D_forecv->c_ys + 1 : 24, 
+		       nwin.histheight))
+    {
+      FreeWindow(p);
+      return -1;
+    }
+#endif
+
 #ifdef KANJI
   p->w_kanji = nwin.kanji;
 #endif
-  ResetWindow(p);	/* sets w_wrap, w_c1, w_gr */
+  ResetWindow(p);	/* sets w_wrap, w_c1, w_gr, w_bce */
 #ifdef FONT
   if (nwin.charset)
     SetCharsets(p, nwin.charset);
@@ -1065,7 +1103,24 @@ char **args, *ttyn;
   int i, pat, wfdused;
   struct pseudowin *pwin = win->w_pwin;
 #endif
+#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
+  int slave;
+#endif
 
+#if defined(sun) && defined(O_NOCTTY)
+  /* sun's utmp_update program opens the salve side, thus corrupting
+   * pty semantics */
+  debug("pre-opening slave...\n");
+  if ((slave = open(ttyn, O_RDWR|O_NOCTTY)) == -1)
+    {
+      Msg(errno, "ttyn");
+      return -1;
+    }
+#endif
+#if defined(_IBMR2) && defined(O_NOCTTY)
+  slave = aixhack;
+  aixhack = -1;
+#endif
   debug("forking...\n");
   proc = *args;
   if (proc == 0)
@@ -1118,14 +1173,14 @@ char **args, *ttyn;
       if (dfp && dfp != stderr)
 	fclose(dfp);
 #endif
-#ifdef _IBMR2
+#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
       close(0);
-      dup(aixhack);
-      close(aixhack);
-#endif
+      dup(slave);
+      close(slave);
+      closeallfiles(win->w_ptyfd);
+      slave = dup(0);
+#else
       closeallfiles(win->w_ptyfd);
-#ifdef _IBMR2
-      aixhack = dup(0);
 #endif
 #ifdef DEBUG
       if (dfp)	/* do not produce child debug, when debug is "off" */
@@ -1187,13 +1242,13 @@ char **args, *ttyn;
       dup(0);
 #endif /* PSEUDOS */
       close(win->w_ptyfd);
-#ifdef _IBMR2
-      close(aixhack);
+#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
+      close(slave);
 #endif
       if (newfd >= 0)
 	{
 	  struct mode fakemode, *modep;
-	  InitPty(newfd);
+	  InitPTY(newfd);
 	  if (fgtty(newfd))
 	    Msg(errno, "fgtty");
 	  if (display)
@@ -1288,9 +1343,8 @@ char **args, *ttyn;
     default:
       break;
     }
-#ifdef _IBMR2
-  close(aixhack);
-  aixhack = -1;
+#if (defined(sun) || defined(_IBMR2)) && defined(O_NOCTTY)
+  close(slave);
 #endif
   return pid;
 }
@@ -1664,7 +1718,7 @@ char *data;
     }
 
   debug1("going to read from window fd %d\n", ev->fd);
-  if ((len = read(ev->fd, buf, size)) <= 0)
+  if ((len = read(ev->fd, buf, size)) < 0)
     {
       if (errno == EINTR || errno == EAGAIN)
 	return;
@@ -1672,7 +1726,13 @@ char *data;
       if (errno == EWOULDBLOCK)
 	return;
 #endif
-      debug2("Window %d: read error (errno %d) - killing window\n", p->w_number, len ? errno : 0);
+      debug2("Window %d: read error (errno %d) - killing window\n", p->w_number, errno);
+      WindowDied(p);
+      return;
+    }
+  if (len == 0)
+    {
+      debug1("Window %d: EOF - killing window\n", p->w_number);
       WindowDied(p);
       return;
     }
@@ -1684,9 +1744,9 @@ char *data;
 	{
 	  debug1("PAKET %x\n", buf[0]);
 	  if (buf[0] & TIOCPKT_NOSTOP)
-	    NewAutoFlow(p, 0);
+	    WNewAutoFlow(p, 0);
 	  if (buf[0] & TIOCPKT_DOSTOP)
-	    NewAutoFlow(p, 1);
+	    WNewAutoFlow(p, 1);
 	}
       bp++;
       len--;
